#
# raccp.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Library General Public License.
#

require 'racc/parser'

module Racc

  class RaccParser < Parser

    attr :start
    attr :classname


    def initialize( racc )
      racc.must Racc::Compiler
      @racc   = racc
      @interf = racc.interf

      @scanner = RaccScanner.new
      @scanner.debug = @yydebug = $RACCPARSER_DEBUG
    end


    def parse( str, fname )
      fname.must String
      @scanner.reset( str )
      @scanner.filename = @filename = fname

      do_parse
    end


    private


    def next_token
      @scanner.scan
    end

    def on_error( tok, val, _values )
      case val
      when Fixnum then v = val.id2name
      when String then v = val
      else
        v = val.inspect
      end
      raise ParseError, <<MES
  parse error:#{@filename}:#{@scanner.lineno}: unexpected token '#{v}'
MES
    end

##### racc version 0.10.6 generates ###

LR_reduce_table = [
0, 0, :racc_error,
6, 20, :_reduce_1,
0, 21, :_reduce_none,
2, 21, :_reduce_none,
3, 23, :_reduce_4,
1, 23, :_reduce_none,
2, 23, :_reduce_6,
2, 24, :_reduce_7,
3, 24, :_reduce_8,
3, 25, :_reduce_9,
3, 25, :_reduce_10,
1, 27, :_reduce_none,
2, 27, :_reduce_none,
2, 28, :_reduce_13,
2, 28, :_reduce_14,
2, 28, :_reduce_15,
1, 29, :_reduce_16,
2, 29, :_reduce_17,
1, 26, :_reduce_18,
1, 26, :_reduce_19,
2, 22, :_reduce_none,
3, 22, :_reduce_none,
3, 30, :_reduce_22,
3, 30, :_reduce_23,
0, 31, :_reduce_24,
2, 31, :_reduce_25,
1, 32, :_reduce_26,
1, 32, :_reduce_27,
1, 32, :_reduce_28,
2, 32, :_reduce_29 ]

LR_reduce_n = 30

LR_shift_n = 52

LR_action_table = [
     2,     2,    -1,   -30,     0,     3,    -1,   -30,     3,     4,
    -1,   -30,     0,     5,    -1,   -30,    -1,    -2,    -1,    52,
     9,     7,    10,     8,     6,    10,     4,    12,     7,    13,
    -1,   -30,    13,    14,    12,    15,    11,    18,    -1,   -30,
    13,    14,    12,    15,    11,    18,    -1,   -30,    -1,    -3,
     3,    23,     8,    22,    -1,   -30,    -1,    -5,     3,    25,
    -1,   -30,     3,    27,    -1,   -30,     3,    23,     8,    22,
    -1,   -30,     3,    23,     8,    22,    -1,   -30,    13,    14,
    12,    15,    10,    31,    11,    18,    -1,   -30,    -1,   -11,
     3,    23,     8,    22,    -1,   -30,     9,    34,    13,    14,
    12,    15,    11,    18,    -1,   -30,     5,    36,     3,    23,
     8,    22,    -1,   -30,     8,    37,    -1,   -30,    -1,   -19,
    -1,   -18,     5,    38,     3,    25,    -1,   -30,    15,    40,
    -1,   -30,    16,    41,    14,    42,    -1,   -30,    -1,    -6,
    -1,   -16,     3,    23,     8,    22,    -1,   -15,     3,    23,
     8,    22,    -1,   -14,    -1,    -9,    -1,   -12,     3,    23,
     8,    22,    -1,   -13,    -1,   -10,     8,    44,    -1,   -30,
    -1,    -4,    -1,    -7,    -1,    -1,    16,    41,    14,    45,
    -1,   -30,    -1,   -24,    -1,   -24,    -1,   -20,    -1,   -17,
    -1,    -8,    -1,   -21,     3,    50,     8,    49,    17,    48,
    18,    51,    -1,   -22,     3,    50,     8,    49,    17,    48,
    18,    51,    -1,   -23,    -1,   -28,    -1,   -27,    -1,   -26,
     3,    23,     8,    22,    -1,   -30,    -1,   -25,    -1,   -29 ]

LR_action_table_ptr = [
     0,     4,     8,    12,    16,    18,    20,    32,    40,    48,
    50,    56,    58,    62,    66,    72,    78,    88,    90,    96,
   106,   114,   118,   120,   122,   128,   132,   138,   140,   142,
   148,   154,   156,   158,   164,   166,   170,   172,   174,   176,
   182,   184,   186,   188,   190,   192,   194,   204,   214,   216,
   218,   220,   226,   228 ]


LR_goto_table = [
    20,     1,    21,     6,    23,     9,    25,    11,    27,    16,
    28,    17,    27,    19,    28,    17,    24,    20,    26,    21,
    22,    24,    30,    26,    26,    28,    29,    29,    26,    28,
    29,    30,    28,    32,    26,    28,    29,    33,    28,    32,
    26,    35,    30,    39,    26,    43,    26,    43,    26,    43,
    31,    46,    31,    47,    32,    52,    32,    52,    26,    53,
    -1,    -1 ]

LR_goto_table_ptr = [
     0,    -1,    -1,    -1,     2,    -1,     4,     8,    12,    -1,
    16,    -1,    20,    -1,    24,    28,    32,    -1,    34,    38,
    40,    -1,    -1,    -1,    42,    -1,    -1,    -1,    -1,    44,
    46,    -1,    -1,    48,    -1,    -1,    -1,    -1,    -1,    -1,
    50,    52,    -1,    -1,    -1,    -1,    54,    56,    -1,    -1,
    -1,    58,    -1,    -1 ]


LR_token_table = {
 false => 0,
 Object.new => 1,
 :CLASS => 2,
 :TOKEN => 3,
 :RULE => 4,
 :XEND => 5,
 :XTOKEN => 6,
 :START => 7,
 :STRING => 8,
 :PRECHIGH => 9,
 :PRECLOW => 10,
 :LEFT => 11,
 :RIGHT => 12,
 :NONASSOC => 13,
 ";" => 14,
 ":" => 15,
 "|" => 16,
 :ACTION => 17,
 "=" => 18 }

DEBUG_PARSER = false

##### racc system variables end #####

 # reduce 0 omitted

 def _reduce_1( val, _values, result )
      @interf.end_register_rule
      @classname = (Integer === val[1]) ? val[1].id2name : val[1]
  result
 end

 # reduce 2 omitted

 # reduce 3 omitted

 def _reduce_4( val, _values, result )
      @interf.end_register_conv
  result
 end

 # reduce 5 omitted

 def _reduce_6( val, _values, result )
      if @start then
        raise ParseError, "start rule defined twice"
      end
      @start = @interf.get_token( val[1] )
  result
 end

 def _reduce_7( val, _values, result )
      @interf.register_conv( val[0], val[1] )
  result
 end

 def _reduce_8( val, _values, result )
      @interf.register_conv( val[1], val[2] )
  result
 end

 def _reduce_9( val, _values, result )
      @interf.end_register_prec( true )
  result
 end

 def _reduce_10( val, _values, result )
      @interf.end_register_prec( false )
  result
 end

 # reduce 11 omitted

 # reduce 12 omitted

 def _reduce_13( val, _values, result )
      @interf.register_prec( :Left, val[1] )
  result
 end

 def _reduce_14( val, _values, result )
      @interf.register_prec( :Right, val[1] )
  result
 end

 def _reduce_15( val, _values, result )
      @interf.register_prec( :Nonassoc, val[1] )
  result
 end

 def _reduce_16( val, _values, result )
      result = val
  result
 end

 def _reduce_17( val, _values, result )
      result.push val[1]
  result
 end

 def _reduce_18( val, _values, result )
      result = @interf.get_token( val[0] )
  result
 end

 def _reduce_19( val, _values, result )
      result = @interf.get_token( eval '"' + val[0] + '"' )
  result
 end

 # reduce 20 omitted

 # reduce 21 omitted

 def _reduce_22( val, _values, result )
      @interf.register_rule @interf.get_token(val[0]), val[2]
  result
 end

 def _reduce_23( val, _values, result )
      @interf.register_rule nil, val[2]
  result
 end

 def _reduce_24( val, _values, result )
      result = []
  result
 end

 def _reduce_25( val, _values, result )
      result.push val[1] if val[1]
  result
 end

 def _reduce_26( val, _values, result )
      result = @interf.get_token( val[0] )
  result
 end

 def _reduce_27( val, _values, result )
      result = @interf.get_token( eval %<"#{result}"> )
  result
 end

 def _reduce_28( val, _values, result )
      result = Action.new( result )
  result
 end

 def _reduce_29( val, _values, result )
      @interf.register_tmpprec val[1]
      result = nil
  result
 end

  end   # class RaccParser

end   # module Racc
